@@ -42,7 +42,6 @@ from tvb.adapters.datatypes.h5.time_series_h5 import TimeSeriesSurfaceH5
 from tvb.adapters.datatypes.db.time_series import TimeSeriesSurfaceIndex
 from tvb.core.neotraits.forms import TraitUploadField, TraitDataTypeSelectField
 from tvb.core.neotraits.db import prepare_array_shape_meta
-from tvb.core.neocom import h5
 from tvb.core.neotraits.uploader_view_model import UploaderViewModel
 from tvb.core.neotraits.view_model import Str, DataTypeGidAttr
 from tvb.datatypes.surfaces import Surface
@@ -105,12 +104,12 @@ class GIFTITimeSeriesImporter(ABCUploader):
         if view_model.surface is None:
             raise LaunchException("No surface selected. Please initiate upload again and select a brain surface.")
 
-        parser = GIFTIParser(self.storage_path, self.operation_id)
+        parser = GIFTIParser(self.operation_id)
         try:
             partial_time_series, gifti_data_arrays = parser.parse(view_model.data_file)
 
             ts_idx = TimeSeriesSurfaceIndex()
-            ts_h5_path = h5.path_for(self.storage_path, TimeSeriesSurfaceH5, ts_idx.gid)
+            ts_h5_path = self.path_for(TimeSeriesSurfaceH5, ts_idx.gid)
 
             ts_h5 = TimeSeriesSurfaceH5(ts_h5_path)
             # todo : make sure that write_time_slice is not required here
