@@ -6,7 +6,7 @@
 # TheVirtualBrain-Scientific Package (for simulators). See content of the
 # documentation-folder for more details. See also http://www.thevirtualbrain.org
 #
-# (c) 2012-2020, Baycrest Centre for Geriatric Care ("Baycrest") and others
+# (c) 2012-2022, Baycrest Centre for Geriatric Care ("Baycrest") and others
 #
 # This program is free software: you can redistribute it and/or modify it under the
 # terms of the GNU General Public License as published by the Free Software Foundation,
@@ -36,6 +36,7 @@ Backend-side for Visualizers that display measures on regions in the brain volum
 
 import json
 from abc import ABCMeta
+
 from six import add_metaclass
 from tvb.adapters.datatypes.db.graph import ConnectivityMeasureIndex
 from tvb.adapters.datatypes.db.region_mapping import RegionVolumeMappingIndex
@@ -43,13 +44,14 @@ from tvb.adapters.datatypes.db.structural import StructuralMRIIndex
 from tvb.adapters.datatypes.db.volume import VolumeIndex
 from tvb.adapters.datatypes.h5.time_series_h5 import TimeSeriesRegionH5
 from tvb.basic.neotraits.api import Attr
-from tvb.core.adapters.arguments_serialisation import *
 from tvb.core.adapters.abcadapter import ABCAdapterForm
 from tvb.core.adapters.abcdisplayer import ABCDisplayer, URLGenerator
+from tvb.core.adapters.arguments_serialisation import *
 from tvb.core.adapters.exceptions import LaunchException
 from tvb.core.entities.filters.chain import FilterChain
 from tvb.core.entities.model.model_datatype import DataTypeMatrix
 from tvb.core.entities.storage import dao
+from tvb.core.neocom import h5
 from tvb.core.neotraits.forms import TraitDataTypeSelectField, StrField
 from tvb.core.neotraits.view_model import ViewModel, DataTypeGidAttr
 from tvb.datatypes.graph import ConnectivityMeasure
@@ -122,16 +124,15 @@ class _MappedArrayVolumeBase(ABCDisplayer):
 
     def get_mapped_array_volume_view(self, entity_gid, mapped_array_gid, x_plane, y_plane, z_plane,
                                      mapped_array_slice=None, **kwargs):
-        entity_h5_class, entity_h5_path = self._load_h5_of_gid(entity_gid)
-        with entity_h5_class(entity_h5_path) as entity_h5:
+
+        with h5.h5_file_for_gid(entity_gid) as entity_h5:
             data_shape = entity_h5.array_data.shape
             x_plane, y_plane, z_plane = preprocess_space_parameters(x_plane, y_plane, z_plane, data_shape[0],
                                                                     data_shape[1], data_shape[2])
             slice_x, slice_y, slice_z = entity_h5.get_volume_slice(x_plane, y_plane, z_plane)
-            connectivity_gid = entity_h5.connectivity.load().hex
+            connectivity_gid = entity_h5.connectivity.load()
 
-        mapped_array_h5_class, mapped_array_h5_path = self._load_h5_of_gid(mapped_array_gid)
-        with mapped_array_h5_class(mapped_array_h5_path) as mapped_array_h5:
+        with h5.h5_file_for_gid(mapped_array_gid) as mapped_array_h5:
             if mapped_array_slice:
                 matrix_slice = parse_slice(mapped_array_slice)
                 measure = mapped_array_h5.array_data[matrix_slice]
@@ -160,11 +161,27 @@ class _MappedArrayVolumeBase(ABCDisplayer):
     def compute_background_params(min_value=0, max_value=0, url=None):
         return dict(minBackgroundValue=min_value, maxBackgroundValue=max_value, urlBackgroundVolumeData=url)
 
+    def get_voxel_region(self, region_mapping_volume_gid, x_plane, y_plane, z_plane):
+        with h5.h5_file_for_gid(region_mapping_volume_gid) as entity_h5:
+            data_shape = entity_h5.array_data.shape
+            x_plane, y_plane, z_plane = preprocess_space_parameters(x_plane, y_plane, z_plane, data_shape[0],
+                                                                    data_shape[1], data_shape[2])
+            slices = slice(x_plane, x_plane + 1), slice(y_plane, y_plane + 1), slice(z_plane, z_plane + 1)
+            voxel = entity_h5.array_data[slices][0, 0, 0]
+            connectivity_gid = entity_h5.connectivity.load()
+
+        if voxel != -1:
+            conn = self.load_traited_by_gid(connectivity_gid)
+            return conn.region_labels[int(voxel)]
+        else:
+            return 'background'
+
     def compute_params(self, region_mapping_volume=None, measure=None, data_slice='', background=None):
         # type: (RegionVolumeMappingIndex, DataTypeMatrix, str, StructuralMRIIndex) -> dict
 
         region_mapping_volume = self._ensure_region_mapping_index(region_mapping_volume, measure)
-        url_voxel_region = URLGenerator.build_h5_url(region_mapping_volume.gid, 'get_voxel_region', parameter='')
+        url_voxel_region = URLGenerator.build_url(self.stored_adapter.id, 'get_voxel_region', region_mapping_volume.gid,
+                                                  parameter='')
 
         if measure is None:
             params = self._compute_region_volume_map_params(region_mapping_volume)
@@ -194,17 +211,35 @@ class _MappedArrayVolumeBase(ABCDisplayer):
         return params
 
     def get_volume_view(self, entity_gid, **kwargs):
-        h5_class, h5_path = self._load_h5_of_gid(entity_gid)
-        with h5_class(h5_path) as h5_opened:
-            if h5_class is TimeSeriesRegionH5:
-                return self._get_volume_view_region(h5_opened, **kwargs)
+        with h5.h5_file_for_gid(entity_gid) as ts_region_h5:
+            if isinstance(ts_region_h5, TimeSeriesRegionH5):
+                return self.prepare_view_region(ts_region_h5, **kwargs)
 
-            return h5_opened.get_volume_view(**kwargs)
+            volume_view = ts_region_h5.get_volume_view(**kwargs)
+        return volume_view
 
-    def _get_volume_view_region(self, ts_h5, from_idx, to_idx, x_plane, y_plane, z_plane, var=0, mode=0):
+    def prepare_view_region(self, ts_h5, x_plane, y_plane, z_plane, from_idx=None, to_idx=None, var=0, mode=0):
+        region_mapping_volume_gid = ts_h5.region_mapping_volume.load()
+
+        if region_mapping_volume_gid is None:
+            raise Exception("Invalid method called for TS without Volume Mapping!")
+
+        with h5.h5_file_for_gid(region_mapping_volume_gid) as volume_rm_h5:
+            volume_rm_shape = volume_rm_h5.array_data.shape
+
+            # Work with space inside Volume:
+            x_plane, y_plane, z_plane = preprocess_space_parameters(x_plane, y_plane, z_plane, volume_rm_shape[0],
+                                                                    volume_rm_shape[1], volume_rm_shape[2])
+            region = self.get_view_region(ts_h5, volume_rm_h5, from_idx, to_idx, x_plane, y_plane, z_plane, var, mode)
+
+        return region
+
+    def get_view_region(self, ts_h5, volume_rm_h5, from_idx, to_idx, x_plane, y_plane, z_plane, var=0, mode=0):
         """
         Retrieve 3 slices through the Volume TS, at the given X, y and Z coordinates, and in time [from_idx .. to_idx].
 
+        :param ts_h5: input TimeSeriesH5
+        :param volume_rm_h5: input RegionVolumeMappingH5
         :param from_idx: int This will be the limit on the first dimension (time)
         :param to_idx: int Also limit on the first Dimension (time)
         :param x_plane: int coordinate
@@ -213,18 +248,6 @@ class _MappedArrayVolumeBase(ABCDisplayer):
 
         :return: An array of 3 Matrices 2D, each containing the values to display in planes xy, yz and xy.
         """
-        region_mapping_volume_gid = ts_h5.region_mapping_volume.load()
-
-        if region_mapping_volume_gid is None:
-            raise Exception("Invalid method called for TS without Volume Mapping!")
-
-        volume_rm_h5_class, volume_rm_h5_path = self._load_h5_of_gid(region_mapping_volume_gid)
-        volume_rm_h5 = volume_rm_h5_class(volume_rm_h5_path)
-        volume_rm_shape = volume_rm_h5.array_data.shape
-
-        # Work with space inside Volume:
-        x_plane, y_plane, z_plane = preprocess_space_parameters(x_plane, y_plane, z_plane, volume_rm_shape[0],
-                                                                volume_rm_shape[1], volume_rm_shape[2])
         var, mode = int(var), int(mode)
         slice_x, slice_y, slice_z = volume_rm_h5.get_volume_slice(x_plane, y_plane, z_plane)
 
@@ -237,8 +260,6 @@ class _MappedArrayVolumeBase(ABCDisplayer):
         regions_ts = ts_h5.read_data_slice(time_slices)[:, 0, :, 0]
         regions_ts = numpy.hstack((regions_ts, numpy.ones((current_time_length, 1)) * ts_h5.out_of_range(min_signal)))
 
-        volume_rm_h5.close()
-
         # Index from TS with the space mapping:
         result_x, result_y, result_z = [], [], []
 
@@ -261,9 +282,9 @@ class BaseVolumeVisualizerModel(ViewModel):
 @add_metaclass(ABCMeta)
 class BaseVolumeVisualizerForm(ABCAdapterForm):
 
-    def __init__(self, prefix='', project_id=None):
-        super(BaseVolumeVisualizerForm, self).__init__(prefix, project_id)
-        self.background = TraitDataTypeSelectField(BaseVolumeVisualizerModel.background, self, name='background')
+    def __init__(self):
+        super(BaseVolumeVisualizerForm, self).__init__()
+        self.background = TraitDataTypeSelectField(BaseVolumeVisualizerModel.background, name='background')
 
 
 class VolumeVisualizerModel(BaseVolumeVisualizerModel):
@@ -288,13 +309,13 @@ class VolumeVisualizerModel(BaseVolumeVisualizerModel):
 
 class VolumeVisualizerForm(BaseVolumeVisualizerForm):
 
-    def __init__(self, prefix='', project_id=None):
-        super(VolumeVisualizerForm, self).__init__(prefix, project_id)
-        self.measure = TraitDataTypeSelectField(VolumeVisualizerModel.measure, self, name='measure',
+    def __init__(self):
+        super(VolumeVisualizerForm, self).__init__()
+        self.measure = TraitDataTypeSelectField(VolumeVisualizerModel.measure, name='measure',
                                                 conditions=self.get_filters())
-        self.region_mapping_volume = TraitDataTypeSelectField(VolumeVisualizerModel.region_mapping_volume, self,
+        self.region_mapping_volume = TraitDataTypeSelectField(VolumeVisualizerModel.region_mapping_volume,
                                                               name='region_mapping_volume')
-        self.data_slice = StrField(VolumeVisualizerModel.data_slice, self, name='data_slice')
+        self.data_slice = StrField(VolumeVisualizerModel.data_slice, name='data_slice')
 
     @staticmethod
     def get_view_model():
@@ -361,13 +382,13 @@ class ConnectivityMeasureVolumeVisualizerModel(BaseVolumeVisualizerModel):
 
 class ConnectivityMeasureVolumeVisualizerForm(BaseVolumeVisualizerForm):
 
-    def __init__(self, prefix='', project_id=None):
-        super(ConnectivityMeasureVolumeVisualizerForm, self).__init__(prefix, project_id)
+    def __init__(self):
+        super(ConnectivityMeasureVolumeVisualizerForm, self).__init__()
         self.connectivity_measure = TraitDataTypeSelectField(
-            ConnectivityMeasureVolumeVisualizerModel.connectivity_measure, self, name='connectivity_measure',
+            ConnectivityMeasureVolumeVisualizerModel.connectivity_measure, name='connectivity_measure',
             conditions=self.get_filters())
         self.region_mapping_volume = TraitDataTypeSelectField(
-            ConnectivityMeasureVolumeVisualizerModel.region_mapping_volume, self, name='region_mapping_volume')
+            ConnectivityMeasureVolumeVisualizerModel.region_mapping_volume, name='region_mapping_volume')
 
     @staticmethod
     def get_view_model():
@@ -430,18 +451,17 @@ class RegionVolumeMappingVisualiserModel(BaseVolumeVisualizerModel):
 
 class RegionVolumeMappingVisualiserForm(BaseVolumeVisualizerForm):
 
-    def __init__(self, prefix='', project_id=None):
-        super(RegionVolumeMappingVisualiserForm, self).__init__(prefix, project_id)
+    def __init__(self):
+        super(RegionVolumeMappingVisualiserForm, self).__init__()
         self.region_mapping_volume = TraitDataTypeSelectField(RegionVolumeMappingVisualiserModel.region_mapping_volume,
-                                                              self, name='region_mapping_volume',
+                                                              name='region_mapping_volume',
                                                               conditions=self.get_filters())
 
         cm_conditions = FilterChain(
             fields=[FilterChain.datatype + '.ndim', FilterChain.datatype + '.has_volume_mapping'],
             operations=["==", "=="], values=[1, True])
         self.connectivity_measure = TraitDataTypeSelectField(RegionVolumeMappingVisualiserModel.connectivity_measure,
-                                                             self, name='connectivity_measure',
-                                                             conditions=cm_conditions)
+                                                             name='connectivity_measure', conditions=cm_conditions)
 
     @staticmethod
     def get_view_model():
@@ -489,8 +509,8 @@ class RegionVolumeMappingVisualiser(_MappedArrayVolumeBase):
 
 class MriVolumeVisualizerForm(BaseVolumeVisualizerForm):
 
-    def __init__(self, prefix='', project_id=None):
-        super(MriVolumeVisualizerForm, self).__init__(prefix, project_id)
+    def __init__(self):
+        super(MriVolumeVisualizerForm, self).__init__()
         self.background.required = True
 
     @staticmethod
