@@ -6,7 +6,7 @@
 # TheVirtualBrain-Scientific Package (for simulators). See content of the
 # documentation-folder for more details. See also http://www.thevirtualbrain.org
 #
-# (c) 2012-2020, Baycrest Centre for Geriatric Care ("Baycrest") and others
+# (c) 2012-2022, Baycrest Centre for Geriatric Care ("Baycrest") and others
 #
 # This program is free software: you can redistribute it and/or modify it under the
 # terms of the GNU General Public License as published by the Free Software Foundation,
@@ -192,13 +192,13 @@ class TopographicViewerModel(ViewModel):
 
 class TopographicViewerForm(ABCAdapterForm):
 
-    def __init__(self, prefix='', project_id=None):
-        super(TopographicViewerForm, self).__init__(prefix, project_id)
-        self.data_0 = TraitDataTypeSelectField(TopographicViewerModel.data_0, self, name='data_0',
+    def __init__(self):
+        super(TopographicViewerForm, self).__init__()
+        self.data_0 = TraitDataTypeSelectField(TopographicViewerModel.data_0, name='data_0',
                                                conditions=self.get_filters())
-        self.data_1 = TraitDataTypeSelectField(TopographicViewerModel.data_1, self, name='data_1',
+        self.data_1 = TraitDataTypeSelectField(TopographicViewerModel.data_1, name='data_1',
                                                conditions=self.get_filters())
-        self.data_2 = TraitDataTypeSelectField(TopographicViewerModel.data_2, self, name='data_2',
+        self.data_2 = TraitDataTypeSelectField(TopographicViewerModel.data_2, name='data_2',
                                                conditions=self.get_filters())
 
     @staticmethod
@@ -236,10 +236,6 @@ class TopographicViewer(ABCDisplayer):
         """
         return -1
 
-    def generate_preview(self, view_model, figure_size=None):
-        # type: (TopographicViewerModel, list) -> dict
-        return self.launch(view_model)
-
     def launch(self, view_model):
         # type: (TopographicViewerModel) -> dict
 
