@@ -40,26 +40,26 @@ import json
 import os
 import sys
 import uuid
-
 import numpy
+
 from tvb.adapters.simulator.simulator_adapter import SimulatorAdapter, CortexViewModel
 from tvb.basic.logger.builder import get_logger
 from tvb.basic.neotraits.api import Range
 from tvb.basic.profile import TvbProfile
 from tvb.core.entities.file.simulator.burst_configuration_h5 import BurstConfigurationH5
 from tvb.core.entities.file.simulator.simulation_history_h5 import SimulationHistory
-from tvb.core.entities.model.db_update_scripts.helper import get_burst_for_migration
+from tvb.core.entities.model.model_burst import BurstConfiguration
 from tvb.core.entities.model.model_datatype import DataTypeGroup
-from tvb.core.entities.storage import dao
+from tvb.core.entities.storage import dao, SA_SESSIONMAKER
 from tvb.core.entities.transient.structure_entities import DataTypeMetaData
 from tvb.core.neocom import h5
 from tvb.core.neocom.h5 import REGISTRY
 from tvb.core.neotraits.h5 import H5File, STORE_STRING, DataSetMetaData
 from tvb.core.services.import_service import OPERATION_XML, ImportService, Operation2ImportData
 from tvb.core.utils import date2string, string2date
-from tvb.datatypes.sensors import SensorTypes
+from tvb.datatypes.sensors import SensorTypesEnum
 from tvb.storage.h5.file.exceptions import MissingDataSetException, IncompatibleFileManagerException, \
-    FileMigrationException, MissingMatlabOctavePathException
+    FileMigrationException
 from tvb.storage.storage_interface import StorageInterface
 
 LOGGER = get_logger(__name__)
@@ -279,19 +279,19 @@ def _migrate_sensors(datasets, **kwargs):
 
 def _migrate_eeg_sensors(**kwargs):
     algorithm, operation_xml_parameters = _migrate_sensors(['labels', 'locations'], **kwargs)
-    operation_xml_parameters['sensors_type'] = SensorTypes.TYPE_EEG.value
+    operation_xml_parameters['sensors_type'] = SensorTypesEnum.TYPE_EEG.value
     return {'algorithm': algorithm, 'operation_xml_parameters': operation_xml_parameters}
 
 
 def _migrate_meg_sensors(**kwargs):
     algorithm, operation_xml_parameters = _migrate_sensors(['labels', 'locations', 'orientations'], **kwargs)
-    operation_xml_parameters['sensors_type'] = SensorTypes.TYPE_MEG.value
+    operation_xml_parameters['sensors_type'] = SensorTypesEnum.TYPE_MEG.value
     return {'algorithm': algorithm, 'operation_xml_parameters': operation_xml_parameters}
 
 
 def _migrate_seeg_sensors(**kwargs):
     algorithm, operation_xml_parameters = _migrate_sensors(['labels', 'locations'], **kwargs)
-    operation_xml_parameters['sensors_type'] = SensorTypes.TYPE_INTERNAL.value
+    operation_xml_parameters['sensors_type'] = SensorTypesEnum.TYPE_INTERNAL.value
     return {'algorithm': algorithm, 'operation_xml_parameters': operation_xml_parameters}
 
 
@@ -1091,8 +1091,8 @@ def _migrate_general_part(input_file):
     root_metadata[TvbProfile.current.version.DATA_VERSION_ATTRIBUTE] = TvbProfile.current.version.DATA_VERSION
 
     # UPDATE CREATION DATE
-    root_metadata[DataTypeMetaData.KEY_DATE] = date2string(string2date(root_metadata[DataTypeMetaData.KEY_DATE],
-                                                                       date_format=DATE_FORMAT_V4_H5))
+    root_metadata[DataTypeMetaData.KEY_DATE] = date2string(
+        string2date(root_metadata[DataTypeMetaData.KEY_DATE], date_format=DATE_FORMAT_V4_H5))
 
     # OBTAIN THE MODULE (for a few data types the old module doesn't exist anymore, in those cases the attr
     # will be set later
@@ -1112,6 +1112,55 @@ def _migrate_general_part(input_file):
     return root_metadata, storage_manager
 
 
+def _get_burst_for_migration(burst_id, burst_match_dict, date_format, selected_db):
+    """
+    This method is supposed to only be used when migrating from version 4 to version 5.
+    It finds a BurstConfig in the old format (when it did not inherit from HasTraitsIndex), deletes it
+    and returns its parameters.
+    """
+    session = SA_SESSIONMAKER()
+    burst_params = session.execute("""SELECT * FROM "BURST_CONFIGURATION" WHERE id = """ + burst_id).fetchone()
+    session.close()
+
+    if burst_params is None:
+        return None, False
+
+    burst_params_dict = {'datatypes_number': burst_params['datatypes_number'],
+                         'dynamic_ids': burst_params['dynamic_ids'], 'range_1': burst_params['range1'],
+                         'range_2': burst_params['range2'], 'fk_project': burst_params['fk_project'],
+                         'name': burst_params['name'], 'status': burst_params['status'],
+                         'error_message': burst_params['error_message'], 'start_time': burst_params['start_time'],
+                         'finish_time': burst_params['finish_time'], 'fk_simulation': burst_params['fk_simulation'],
+                         'fk_operation_group': burst_params['fk_operation_group'],
+                         'fk_metric_operation_group': burst_params['fk_metric_operation_group']}
+
+    if selected_db == 'sqlite':
+        burst_params_dict['start_time'] = string2date(burst_params_dict['start_time'], date_format=date_format)
+        burst_params_dict['finish_time'] = string2date(burst_params_dict['finish_time'], date_format=date_format)
+
+    if burst_id not in burst_match_dict:
+        burst_config = BurstConfiguration(burst_params_dict['fk_project'])
+        burst_config.datatypes_number = burst_params_dict['datatypes_number']
+        burst_config.dynamic_ids = burst_params_dict['dynamic_ids']
+        burst_config.error_message = burst_params_dict['error_message']
+        burst_config.finish_time = burst_params_dict['finish_time']
+        burst_config.fk_metric_operation_group = burst_params_dict['fk_metric_operation_group']
+        burst_config.fk_operation_group = burst_params_dict['fk_operation_group']
+        burst_config.fk_project = burst_params_dict['fk_project']
+        burst_config.fk_simulation = burst_params_dict['fk_simulation']
+        burst_config.name = burst_params_dict['name']
+        burst_config.range1 = burst_params_dict['range_1']
+        burst_config.range2 = burst_params_dict['range_2']
+        burst_config.start_time = burst_params_dict['start_time']
+        burst_config.status = burst_params_dict['status']
+        new_burst = True
+    else:
+        burst_config = dao.get_burst_by_id(burst_match_dict[burst_id])
+        new_burst = False
+
+    return burst_config, new_burst
+
+
 def update(input_file, burst_match_dict):
     """
     :param input_file: the file that needs to be converted to a newer file storage version.
@@ -1124,8 +1173,9 @@ def update(input_file, burst_match_dict):
     split_path = input_file.split(os.path.sep)
     storage_migrate = True
     try:
-        # Change file names only for storage migration
         op_id = int(split_path[-2])
+
+        # Change file names only for storage migration
         file_basename = os.path.basename(input_file)
         replaced_basename = file_basename.replace('-', '')
         replaced_basename = replaced_basename.replace('BrainSkull', 'Surface')
@@ -1164,12 +1214,11 @@ def update(input_file, burst_match_dict):
 
     try:
         # Take information out from the Operation.xml file
-        if OPERATION_XML in files_in_folder:
+        if op_id is not None and OPERATION_XML in files_in_folder:
             operation_file_path = os.path.join(folder, OPERATION_XML)
-            project = dao.get_project_by_name(split_path[-3])
-            xml_operation, operation_xml_parameters, algorithm = \
-                import_service.build_operation_from_file(project, operation_file_path)
             operation = dao.get_operation_by_id(op_id)
+            xml_operation, operation_xml_parameters, algorithm = \
+                import_service.build_operation_from_file(operation.project, operation_file_path)
             try:
                 operation_xml_parameters = json.loads(operation_xml_parameters)
             except NameError:
@@ -1230,9 +1279,7 @@ def update(input_file, burst_match_dict):
             algorithm = dao.get_algorithm_by_module(alg_json['module'], alg_json['classname'])
 
             if algorithm is None:
-                raise MissingMatlabOctavePathException(alg_json['classname'] + 'data file could not be migrated.'
-                                                                               ' Please specify a valid path to locally'
-                                                                               ' installed Matlab or Octave!')
+                raise FileMigrationException(alg_json['classname'] + 'data file could not be migrated.')
             operation.algorithm = algorithm
             operation.fk_from_algo = algorithm.id
 
@@ -1242,10 +1289,10 @@ def update(input_file, burst_match_dict):
             vm = import_service.create_view_model(operation, operation_data, folder,
                                                   generic_attributes, additional_params)
 
-            if 'TimeSeries' in class_name and 'Importer' not in operation_entity.algorithm.classname \
+            if 'TimeSeries' in class_name and 'Importer' not in algorithm.classname \
                     and time_series_gid is None:
-                burst_config, new_burst = get_burst_for_migration(possible_burst_id, burst_match_dict,
-                                                                  DATE_FORMAT_V4_DB, TvbProfile.current.db.SELECTED_DB)
+                burst_config, new_burst = _get_burst_for_migration(possible_burst_id, burst_match_dict,
+                                                                   DATE_FORMAT_V4_DB, TvbProfile.current.db.SELECTED_DB)
                 if burst_config:
                     root_metadata['parent_burst'] = _parse_gid(burst_config.gid)
                     burst_config.simulator_gid = vm.gid.hex
@@ -1265,7 +1312,8 @@ def update(input_file, burst_match_dict):
                             alg.configure()
                             simulation_history = SimulationHistory()
                             simulation_history.populate_from(alg)
-                            history_index = h5.store_complete(simulation_history, folder,
+                            history_index = h5.store_complete(simulation_history, op_id,
+                                                              operation.project.name,
                                                               generic_attributes=vm.generic_attributes)
                             history_index.fk_from_operation = op_id
                             history_index.fk_parent_burst = burst_config.gid
