@@ -6,7 +6,7 @@
 # TheVirtualBrain-Scientific Package (for simulators). See content of the
 # documentation-folder for more details. See also http://www.thevirtualbrain.org
 #
-# (c) 2012-2020, Baycrest Centre for Geriatric Care ("Baycrest") and others
+# (c) 2012-2022, Baycrest Centre for Geriatric Care ("Baycrest") and others
 #
 # This program is free software: you can redistribute it and/or modify it under the
 # terms of the GNU General Public License as published by the Free Software Foundation,
@@ -28,19 +28,16 @@
 #
 #
 
-import os
 import json
 import uuid
 from collections import namedtuple
-from datetime import datetime
 import numpy
-from tvb.core import utils
-from tvb.core.entities.file.files_helper import FilesHelper
-from tvb.core.entities.storage import dao
+
 from tvb.core.entities.filters.chain import FilterChain
 from tvb.core.neocom.h5 import REGISTRY
 from tvb.basic.neotraits.ex import TraitError
-from tvb.basic.neotraits.api import List, Attr
+from tvb.basic.neotraits.api import List, Attr, TVBEnum
+# TODO: remove dependency
 from tvb.core.neotraits.db import HasTraitsIndex
 from tvb.core.neotraits.view_model import DataTypeGidAttr
 
@@ -51,20 +48,12 @@ from tvb.core.neotraits.view_model import DataTypeGidAttr
 jinja_env = None
 
 
-def prepare_prefixed_name_for_field(prefix, name):
-    if prefix != "":
-        return '{}_{}'.format(prefix, name)
-    else:
-        return name
-
-
 class Field(object):
     template = None
 
-    def __init__(self, form, name, disabled=False, required=False, label='', doc='', default=None):
-        # type: (Form, str, bool, bool, str, str, object) -> None
-        self.owner = form
-        self.name = prepare_prefixed_name_for_field(form.prefix, name)
+    def __init__(self, name, disabled=False, required=False, label='', doc='', default=None):
+        # type: (str, bool, bool, str, str, object) -> None
+        self.name = name
         self.disabled = disabled
         self.required = required
         self.label = label
@@ -76,7 +65,6 @@ class Field(object):
         # keeps the deserialized data
         self.data = None
         # keeps user input, even if wrong, we have to redisplay it
-        # todo
         self.unvalidated_data = default
         self.errors = []
 
@@ -93,7 +81,7 @@ class Field(object):
         return not self.errors
 
     def _from_post(self):
-        if self.required and self.unvalidated_data is None:
+        if self.required and (self.unvalidated_data is None or len(str(self.unvalidated_data).strip()) == 0):
             raise ValueError('Field required')
         self.data = self.unvalidated_data
 
@@ -110,47 +98,14 @@ class Field(object):
         return jinja_env.get_template(self.template).render(field=self)
 
 
-class SimpleStrField(Field):
-    template = 'form_fields/str_field.html'
-
-    def _from_post(self):
-        if self.required and (self.unvalidated_data is None or self.unvalidated_data.strip() == ''):
-            raise ValueError('Field required')
-        self.data = self.unvalidated_data
-
-
-class SimpleHiddenField(Field):
-    template = 'form_fields/hidden_field.html'
-
-
-class SimpleFloatField(Field):
-    template = 'form_fields/number_field.html'
-    input_type = "number"
-    min = None
-    max = None
-    step = 'any'
-
-    def _from_post(self):
-        super(SimpleFloatField, self)._from_post()
-        if self.unvalidated_data and len(self.unvalidated_data) == 0:
-            self.unvalidated_data = None
-        if self.unvalidated_data:
-            self.data = float(self.unvalidated_data)
-        else:
-            self.data = None
-
-
 class TraitField(Field):
-    # mhtodo: while this is consistent with the h5 api, it has the same problem
-    #         it couples the system to traited attr declarations
-    def __init__(self, trait_attribute, form, name=None, disabled=False):
-        # type: (Attr, Form, str, bool) -> None
+    def __init__(self, trait_attribute, name=None, disabled=False):
+        # type: (Attr, str, bool) -> None
         self.trait_attribute = trait_attribute  # type: Attr
         name = name or trait_attribute.field_name
         label = trait_attribute.label or name
 
         super(TraitField, self).__init__(
-            form,
             name,
             disabled,
             trait_attribute.required,
@@ -169,47 +124,20 @@ TEMPORARY_PREFIX = ".tmp"
 class TraitUploadField(TraitField):
     template = 'form_fields/upload_field.html'
 
-    def __init__(self, traited_attribute, required_type, form, name, disabled=False):
-        super(TraitUploadField, self).__init__(traited_attribute, form, name, disabled)
+    def __init__(self, traited_attribute, required_type, name, disabled=False):
+        super(TraitUploadField, self).__init__(traited_attribute, name, disabled)
         self.required_type = required_type
-        self.files_helper = FilesHelper()
-
-    def fill_from_post(self, post_data):
-        super(TraitUploadField, self).fill_from_post(post_data)
-
-        if self.data.file is None:
-            self.data = None
-            return
-
-        project = dao.get_project_by_id(self.owner.project_id)
-        temporary_storage = self.files_helper.get_project_folder(project, self.files_helper.TEMP_FOLDER)
-
-        file_name = None
-        try:
-            uq_name = utils.date2string(datetime.now(), True) + '_' + str(0)
-            file_name = TEMPORARY_PREFIX + uq_name + '_' + self.data.filename
-            file_name = os.path.join(temporary_storage, file_name)
-
-            with open(file_name, 'wb') as file_obj:
-                file_obj.write(self.data.file.read())
-        except Exception as excep:
-            # TODO: is this handled properly?
-            self.files_helper.remove_files([file_name])
-            excep.message = 'Could not continue: Invalid input files'
-            raise excep
-
-        if file_name:
-            self.data = file_name
-            self.owner.temporary_files.append(file_name)
 
 
 class TraitDataTypeSelectField(TraitField):
     template = 'form_fields/datatype_select_field.html'
     missing_value = 'explicit-None-value'
 
-    def __init__(self, trait_attribute, form, name=None, conditions=None, draw_dynamic_conditions_buttons=True,
-                 dynamic_conditions=None, has_all_option=False, show_only_all_option=False):
-        super(TraitDataTypeSelectField, self).__init__(trait_attribute, form, name)
+    def __init__(self, trait_attribute, name=None, conditions=None,
+                 draw_dynamic_conditions_buttons=True, has_all_option=False,
+                 show_only_all_option=False):
+        super(TraitDataTypeSelectField, self).__init__(trait_attribute, name)
+
         if issubclass(type(trait_attribute), DataTypeGidAttr):
             type_to_query = trait_attribute.linked_datatype
         else:
@@ -221,9 +149,9 @@ class TraitDataTypeSelectField(TraitField):
             self.datatype_index = REGISTRY.get_index_for_datatype(type_to_query)
         self.conditions = conditions
         self.draw_dynamic_conditions_buttons = draw_dynamic_conditions_buttons
-        self.dynamic_conditions = dynamic_conditions
         self.has_all_option = has_all_option
         self.show_only_all_option = show_only_all_option
+        self.datatype_options = []
 
     def from_trait(self, trait, f_name):
         if hasattr(trait, f_name):
@@ -239,21 +167,7 @@ class TraitDataTypeSelectField(TraitField):
     def get_form_filters(self):
         return self.conditions
 
-    def _get_values_from_db(self):
-        all_conditions = FilterChain()
-        all_conditions += self.conditions
-        all_conditions += self.dynamic_conditions
-        filtered_datatypes, count = dao.get_values_of_datatype(self.owner.project_id,
-                                                               self.datatype_index,
-                                                               all_conditions)
-        return filtered_datatypes
-
     def options(self):
-        if not self.owner.project_id:
-            raise ValueError('A project_id is required in order to query the DB')
-
-        filtered_datatypes = self._get_values_from_db()
-
         if not self.required:
             choice = None
             yield Option(
@@ -264,21 +178,19 @@ class TraitDataTypeSelectField(TraitField):
             )
 
         if not self.show_only_all_option:
-            for i, datatype in enumerate(filtered_datatypes):
+            for i, dt_opt in enumerate(self.datatype_options):
                 yield Option(
                     id='{}_{}'.format(self.name, i),
-                    value=datatype[2],
-                    label=self._prepare_display_name(datatype),
-                    checked=self.data == datatype[2]
+                    value=dt_opt[0][2],
+                    label=dt_opt[1],
+                    checked=self.data == dt_opt[0][2]
                 )
 
         if self.has_all_option:
-            if not self.owner.draw_ranges:
-                raise ValueError("The owner form should draw ranges inputs in order to support 'All' option")
 
             all_values = ''
-            for fdt in filtered_datatypes:
-                all_values += str(fdt[2]) + ','
+            for fdt in self.datatype_options:
+                all_values += str(fdt[0][2]) + ','
 
             choice = "All"
             yield Option(
@@ -288,33 +200,6 @@ class TraitDataTypeSelectField(TraitField):
                 checked=self.data is choice
             )
 
-    def get_dt_from_db(self):
-        return dao.get_datatype_by_gid(self.data)
-
-    def _prepare_display_name(self, value):
-        """
-        Populate meta-data fields for data_list (list of DataTypes).
-
-        Private method, to be called recursively.
-        It will receive a list of Attributes, and it will populate 'options'
-        entry with data references from DB.
-        """
-        # Here we only populate with DB data, actual
-        # XML check will be done after select and submit.
-        entity_gid = value[2]
-        actual_entity = dao.get_generic_entity(self.datatype_index, entity_gid, "gid")
-        display_name = actual_entity[0].display_name
-        display_name += ' - ' + (value[3] or "None ")  # Subject
-        if value[5]:
-            display_name += ' - From: ' + str(value[5])
-        else:
-            display_name += utils.date2string(value[4])
-        if value[6]:
-            display_name += ' - ' + str(value[6])
-        display_name += ' - ID:' + str(value[0])
-
-        return display_name
-
     def _from_post(self):
         if self.unvalidated_data == self.missing_value:
             self.unvalidated_data = None
@@ -335,19 +220,14 @@ class TraitDataTypeSelectField(TraitField):
 class StrField(TraitField):
     template = 'form_fields/str_field.html'
 
-    def _from_post(self):
-        if self.required and (self.unvalidated_data is None or self.unvalidated_data.strip() == ''):
-            raise ValueError('Field required')
-        self.data = self.unvalidated_data
 
-
-class BytesField(StrField):
-    """ StrField for byte strings. """
+class UserSessionStrField(TraitField):
     template = 'form_fields/str_field.html'
 
-    def _from_post(self):
-        super(BytesField, self)._from_post()
-        self.data = self.unvalidated_data.encode('utf-8')
+    def __init__(self, trait_attribute, name=None, disabled=False, key=None):
+        # type: (Attr, str, bool, str) -> None
+        self.key = key # Key in session
+        super(UserSessionStrField, self).__init__(trait_attribute, name, disabled)
 
 
 class BoolField(TraitField):
@@ -364,44 +244,41 @@ class IntField(TraitField):
 
     def _from_post(self):
         super(IntField, self)._from_post()
-        if self.unvalidated_data and len(self.unvalidated_data) == 0:
-            self.unvalidated_data = None
-        if self.unvalidated_data:
-            self.data = int(self.unvalidated_data)
-        else:
+        if len(self.unvalidated_data.strip()) == 0:
             self.data = None
+        else:
+            self.data = int(self.unvalidated_data)
 
 
 class FloatField(TraitField):
     template = 'form_fields/number_field.html'
-    input_type = "number"
     min = None
     max = None
     step = 'any'
 
     def _from_post(self):
         super(FloatField, self)._from_post()
-        if self.unvalidated_data and len(self.unvalidated_data) == 0:
-            self.unvalidated_data = None
-        if self.unvalidated_data:
-            self.data = float(self.unvalidated_data)
-        else:
+        if len(self.unvalidated_data.strip()) == 0:
             self.data = None
+        else:
+            self.data = float(self.unvalidated_data)
 
 
 class ArrayField(TraitField):
     template = 'form_fields/str_field.html'
 
     def _from_post(self):
+        super(ArrayField, self)._from_post()
         self.data = None
-        if self.unvalidated_data is not None:
+        if len(self.unvalidated_data.strip()) == 0:
+            self.data = None
+        else:
             data = json.loads(self.unvalidated_data)
             self.data = numpy.array(data, dtype=self.trait_attribute.dtype)
 
     @property
     def value(self):
         if self.data is None:
-            # todo: maybe we need to distinguish None from missing data
             # this None means self.data is missing, either not set or unset cause of validation error
             return self.unvalidated_data
         try:
@@ -426,21 +303,22 @@ class SelectField(TraitField):
         if len(choices) > 4:
             self.template = 'form_fields/select_field.html'
 
-    def __init__(self, trait_attribute, form, name=None, disabled=False, choices=None, display_none_choice=True,
-                 subform=None, display_subform=True):
-        super(SelectField, self).__init__(trait_attribute, form, name, disabled)
-        if choices:
-            self.choices = choices
-        else:
-            self.choices = {choice: choice for choice in trait_attribute.choices}
+    def __init__(self, trait_attribute, name=None, disabled=False, display_none_choice=True,
+                 subform=None, display_subform=True, ui_values=None, session_key=None, form_key=None):
+        super(SelectField, self).__init__(trait_attribute, name, disabled)
+
+        self.choices = list(trait_attribute.choices)
         if not self.choices:
             raise ValueError('no choices for field')
         self.display_none_choice = display_none_choice
         self.subform_field = None
         if subform:
-            self.subform_field = FormField(subform, form, self.subform_prefix + self.name)
+            self.subform_field = FormField(subform, self.subform_prefix + self.name)
             self.display_subform = display_subform
         self._prepare_template(self.choices)
+        self.ui_values = ui_values
+        self.session_key = session_key
+        self.form_key = form_key
 
     @property
     def value(self):
@@ -460,45 +338,100 @@ class SelectField(TraitField):
                     checked=self.data is None
                 )
 
+        choices = self.choices if self.ui_values is None else self.ui_values
+
         for i, choice in enumerate(self.choices):
             yield Option(
                 id='{}_{}'.format(self.name, i),
                 value=choice,
-                label=str(choice).title(),
-                checked=self.value == self.choices.get(choice)
+                label=str(choices[i]).title(),
+                checked=self.value == choice or (self.value == choice.value if hasattr(choice, 'value') else False)
             )
 
-    # def _from_post(self):
-    #     # encode None as a string
-    #     if self.unvalidated_data == self.missing_value:
-    #         self.unvalidated_data = None
-    #
-    #     if self.required and not self.unvalidated_data:
-    #         raise ValueError('Field required')
-    #
-    #     if self.unvalidated_data is not None:
-    #         # todo muliple values
-    #         self.data = self.trait_attribute.field_type(self.unvalidated_data)
-    #     else:
-    #         self.data = None
-    #
-    #     allowed = self.trait_attribute.choices
-    #     if not self.trait_attribute.required:
-    #         allowed = (None,) + allowed
-    #
-    #     if self.data not in allowed:
-    #         raise ValueError('must be one of {}'.format(allowed))
+    def _from_post(self):
+        super(SelectField, self)._from_post()
 
-    def fill_from_post(self, post_data):
-        super(SelectField, self).fill_from_post(post_data)
-        self.data = self.choices.get(self.data)
+        data_as_enum = TVBEnum.string_to_enum(self.choices, self.unvalidated_data)
+        if self.unvalidated_data != self.missing_value and data_as_enum is None \
+                and (self.unvalidated_data is not None or self.display_none_choice is False):
+            raise ValueError("the entered value is not among the choices for this field!")
+
+        self.data = data_as_enum
+
+
+class DynamicSelectField(TraitField):
+    template = 'form_fields/radio_field.html'
+    missing_value = 'explicit-None-value'
+    subform_prefix = 'subform_'
+
+    def _prepare_template(self, choices):
+        if len(choices) > 4:
+            self.template = 'form_fields/select_field.html'
+
+    def __init__(self, trait_attribute, name=None, disabled=False, choices=None, display_none_choice=True,
+                 subform=None, display_subform=True, ui_values=None):
+        super(DynamicSelectField, self).__init__(trait_attribute, name, disabled)
+        self.choices = choices
+        if not self.choices:
+            raise ValueError('no choices for field')
+        self.display_none_choice = display_none_choice
+        self.subform_field = None
+        if subform:
+            self.subform_field = FormField(subform, self.subform_prefix + self.name)
+            self.display_subform = display_subform
+        self._prepare_template(self.choices)
+        self.ui_values = ui_values
+
+    @property
+    def value(self):
+        if self.data is None and not self.trait_attribute.required:
+            return self.data
+        return super(DynamicSelectField, self).value
+
+    def options(self):
+        """ to be used from template, assumes self.data is set """
+        if self.display_none_choice:
+            if not self.trait_attribute.required:
+                choice = None
+                yield Option(
+                    id='{}_{}'.format(self.name, None),
+                    value=self.missing_value,
+                    label=str(choice).title(),
+                    checked=self.data is None
+                )
+
+        choices = self.choices if self.ui_values is None else self.ui_values
+
+        for i, choice in enumerate(self.choices):
+            yield Option(
+                id='{}_{}'.format(self.name, i),
+                value=str(choice),
+                label=str(choices[i]),
+                checked=self.value == choice
+            )
+
+    def _from_post(self):
+        super(DynamicSelectField, self)._from_post()
+
+        data_as_object = self.__string_to_object()
+        if self.unvalidated_data != self.missing_value and data_as_object is None \
+                and (self.unvalidated_data is not None or self.display_none_choice is False):
+            raise ValueError("the entered value is not among the choices for this field!")
+
+        self.data = data_as_object
+
+    def __string_to_object(self):
+        for choice in self.choices:
+            if self.unvalidated_data == str(choice):
+                return choice
+        return None
 
 
 class MultiSelectField(TraitField):
     template = 'form_fields/checkbox_field.html'
 
-    def __init__(self, trait_attribute, form, name=None, disabled=False):
-        super(MultiSelectField, self).__init__(trait_attribute, form, name, disabled)
+    def __init__(self, trait_attribute, name=None, disabled=False):
+        super(MultiSelectField, self).__init__(trait_attribute, name, disabled)
         if not isinstance(trait_attribute, List):
             raise NotImplementedError('only List in multi select for now')
 
@@ -535,36 +468,31 @@ class MultiSelectField(TraitField):
         self.data = data
 
 
-# noinspection PyPep8Naming
-def ScalarField(trait_attribute, form, name=None, disabled=False):
-    # as this makes introspective decisions it has to be moved at a different level
-    field_type_for_trait_type = {
-        # str: BytesField,
-        str: StrField,
-        int: IntField,
-        float: FloatField,
-        bool: BoolField,
-    }
+class HiddenField(TraitField):
+    template = 'form_fields/hidden_field.html'
 
-    if trait_attribute.choices is not None:
-        cls = SelectField
-    else:
-        cls = field_type_for_trait_type.get(trait_attribute.field_type)
+    def __init__(self, trait_attribute, name=None, disabled=False):
+        super(HiddenField, self).__init__(trait_attribute, name, disabled)
+        self.label = ''
 
-    if isinstance(trait_attribute, List) and trait_attribute.element_choices:
-        cls = MultiSelectField
 
-    if cls is None:
-        raise ValueError('can not make a scalar field for trait attribute {}'.format(trait_attribute))
+class LabelField(TraitField):
+    template = 'form_fields/label_field.html'
 
-    return cls(trait_attribute, form, name=name, disabled=disabled)
+    def __init__(self, trait_attribute, label_message=None, name=None):
+        super(LabelField, self).__init__(trait_attribute, name, disabled=True)
+
+        if label_message is None:
+            self.label_message = trait_attribute.label
+        else:
+            self.label_message = label_message
 
 
 class FormField(Field):
     template = 'form_fields/form_field.html'
 
-    def __init__(self, form_class, form, name, label='', doc=''):
-        super(FormField, self).__init__(form, name, False, False, label, doc)
+    def __init__(self, form_class, name, label='', doc=''):
+        super(FormField, self).__init__(name, False, False, label, doc)
         self.form = form_class()
 
     def fill_from_post(self, post_data):
@@ -574,30 +502,19 @@ class FormField(Field):
     def validate(self):
         return self.form.validate()
 
-    @property
-    def prefix_name(self):
-        return self.form.prefix
-
     def __str__(self):
         return jinja_env.get_template(self.template).render(adapter_form=self.form)
 
 
 class Form(object):
-    RANGE_1_NAME = 'range_1'
-    RANGE_2_NAME = 'range_2'
-    range_1 = None
-    range_2 = None
 
-    def __init__(self, prefix='', project_id=None, draw_ranges=True):
-        # TODO: makes sense here?
-        self.project_id = project_id
-        self.prefix = prefix
+    def __init__(self):
         self.errors = []
-        self.draw_ranges = draw_ranges
 
-    def get_subform_key(self):
+    @staticmethod
+    def get_subform_key():
         """
-        If the current form can be used as subform, this method should return the proper value from SubformsEnum.
+        If the current form can be used as subform, this method should return the proper value.
         """
         raise NotImplementedError
 
@@ -660,11 +577,11 @@ class Form(object):
                 field.errors.append(ex)
                 raise
 
-    def fill_trait_partially(self, datatype, fields = None):
+    def fill_trait_partially(self, datatype, fields=None):
         for field in self.trait_fields:
             f_name = field.trait_attribute.field_name
             if f_name is None or \
-                    fields is not None and f_name not in fields :
+                    fields is not None and f_name not in fields:
                 # skipp attribute that does not seem to belong to a traited type
                 continue
             try:
@@ -679,8 +596,6 @@ class Form(object):
     def fill_from_post(self, form_data):
         for field in self.fields:
             field.fill_from_post(form_data)
-        self.range_1 = form_data.get(self.RANGE_1_NAME)
-        self.range_2 = form_data.get(self.RANGE_2_NAME)
 
     def fill_from_single_post_param(self, **param):
         param_key = list(param)[0]
